@@ -35,7 +35,8 @@ from sklearn.tree import DecisionTreeClassifier, ExtraTreeClassifier
 from sklearn.svm import NuSVC
 from xgboost import XGBClassifier
 from imblearn.ensemble import BalancedBaggingClassifier
-from MultiTrain.methods.multitrain_methods import directory, img, img_plotly, kf_best_model, write_to_excel
+from MultiTrain.methods.multitrain_methods import directory, img, img_plotly, kf_best_model, write_to_excel, \
+    _check_target
 from skopt import BayesSearchCV
 from sklearn.model_selection import train_test_split, GridSearchCV, RandomizedSearchCV, cross_validate
 from sklearn.experimental import enable_halving_search_cv  # noqa
@@ -53,7 +54,8 @@ import time
 
 import logging
 import os
-#os.environ['OMP_NUM_THREADS'] = "1"
+
+# os.environ['OMP_NUM_THREADS'] = "1"
 
 logging.basicConfig(level=logging.ERROR)
 logger = logging.getLogger(__name__)
@@ -66,7 +68,6 @@ class MultiClassifier:
                  cores: int = -1,
                  random_state: int = randint(1000),
                  verbose: bool = False,
-                 target_class: str = 'binary',
                  imbalanced: bool = False,
                  sampling: str = None,
                  strategy: str or float = "auto") -> None:
@@ -74,12 +75,10 @@ class MultiClassifier:
         self.cores = cores
         self.random_state = random_state
         self.verbose = verbose
-        self.target_class = target_class
         self.sampling = sampling
         self.imbalanced = imbalanced
         self.strategy = strategy
-        # self.param_tuning = param_tuning
-        # self.timeout = timeout
+
         self.oversampling_list = ['SMOTE', 'RandomOverSampler', 'SMOTEN', 'ADASYN',
                                   'BorderlineSMOTE', 'KMeansSMOTE', 'SVMSMOTE']
         self.oversampling_methods = [SMOTE(sampling_strategy=self.strategy, random_state=self.random_state),
@@ -101,18 +100,21 @@ class MultiClassifier:
                                                               n_jobs=self.cores),
                                       RepeatedEditedNearestNeighbours(sampling_strategy=self.strategy,
                                                                       n_jobs=self.cores),
-                                      AllKNN(sampling_strategy=self.strategy, n_jobs=self.cores),
+                                      AllKNN(sampling_strategy=self.strategy,
+                                             n_jobs=self.cores),
                                       InstanceHardnessThreshold(sampling_strategy=self.strategy,
                                                                 random_state=self.random_state,
                                                                 n_jobs=self.cores),
-                                      NearMiss(sampling_strategy=self.strategy, n_jobs=self.cores),
+                                      NearMiss(sampling_strategy=self.strategy,
+                                               n_jobs=self.cores),
                                       NeighbourhoodCleaningRule(sampling_strategy=self.strategy,
                                                                 n_jobs=self.cores),
                                       OneSidedSelection(sampling_strategy=self.strategy,
                                                         n_jobs=self.cores),
                                       RandomUnderSampler(sampling_strategy=self.strategy,
                                                          random_state=self.random_state),
-                                      TomekLinks(sampling_strategy=self.strategy, n_jobs=self.cores)]
+                                      TomekLinks(sampling_strategy=self.strategy,
+                                                 n_jobs=self.cores)]
 
         self.over_under_list = ['SMOTEENN', 'SMOTETomek']
         self.over_under_methods = [SMOTEENN(sampling_strategy=self.strategy,
@@ -190,6 +192,9 @@ class MultiClassifier:
               normalize: any = None,
               columns_to_scale: list = None,
               n_components: int = None):
+
+        global the_y
+        the_y = y
         """
         :param X: features
         :param y: labels
@@ -321,7 +326,7 @@ class MultiClassifier:
         """
         It initializes all the models that we will be using in our ensemble
         """
-        #if self.param_tuning is True:
+        # if self.param_tuning is True:
         #    if self.timeout is False:
         #        logger.info('It is recommended to set a timeout to avoid longer training times')
 
@@ -385,8 +390,8 @@ class MultiClassifier:
 
     def _startKFold_(self, param, param_X, param_y, param_cv, train_score):
         names = self.classifier_model_names()
-
-        if self.target_class == 'binary':
+        target_class = _check_target(param_y)
+        if target_class == 'binary':
             dataframe = {}
             for i in range(len(param)):
 
@@ -442,7 +447,7 @@ class MultiClassifier:
                     dataframe.update({names[i]: scores_df})
             return dataframe
 
-        elif self.target_class == 'multiclass':
+        elif target_class == 'multiclass':
             dataframe = {}
             for j in range(len(param)):
                 start = time.time()
@@ -495,7 +500,7 @@ class MultiClassifier:
             show_train_score: bool = False,
             text: bool = False,
             vectorizer: str = None,
-            ngrams: tuple = None
+            ngrams: tuple = None,
             ) -> DataFrame:
         # If splitting is False, then do nothing. If splitting is True, then assign the values of split_data to the
         # variables X_train, X_test, y_train, and y_test
@@ -543,6 +548,8 @@ class MultiClassifier:
         # fit(X = features, y = labels, kf = True, fold = (10, 42, True))
 
         global y_te
+
+        target_class = _check_target(y) if y else _check_target(split_data[3])
         if text:
             if isinstance(text, bool) is False:
                 raise TypeError('parameter text is of type bool only. set to true or false')
@@ -590,13 +597,6 @@ class MultiClassifier:
         if kf is True and (X is None or y is None or (X is None and y is None)):
             raise ValueError("Set the values of features X and target y")
 
-        if self.target_class:
-            accepted_targets = ['binary', 'multiclass']
-            if self.target_class not in accepted_targets:
-                raise Exception(
-                    f"target_class should be set to either binary or multiclass but target_class was set to "
-                    f"{self.target_class}")
-
         if splitting is True or split_self is True:
             if splitting and split_data:
                 X_tr, X_te, y_tr, y_te = split_data[0], split_data[1], split_data[2], split_data[3]
@@ -605,10 +605,7 @@ class MultiClassifier:
                     and y_train is not None \
                     and y_test is not None:
                 X_tr, X_te, y_tr, y_te = X_train, X_test, y_train, y_test
-            print('initializing')
             model = self._initialize_()
-            print('done')
-            print('getting model names')
             names = self.classifier_model_names()
             dataframe = {}
             for i in range(len(model)):
@@ -622,7 +619,7 @@ class MultiClassifier:
                         try:
                             model[i].fit(X_tr, y_tr)
                         except ValueError:
-                            logger.error(f'{model[i]} has an issue')
+                            logger.error(f'{model[i]} unable to fit properly')
                             pass
 
                     elif self.imbalanced is True:
@@ -703,33 +700,28 @@ class MultiClassifier:
                                 pipeline.fit(X_tr, y_tr)
 
                         except Exception:
-                            logger.error(f'{model[i]} has an issue')
+                            logger.error(f'{model[i]} cannot fit properly')
                             pass
 
                     end = time.time()
 
-                    # pred = pipeline.predict(X_te)
-                    # pred_train = pipeline.predict(X_tr)
-
                 true = y_te
                 true_train = y_tr
-                # print(f'{model[i]}: {accuracy_score(true, pred)}')
+
                 acc = accuracy_score(true, pred)
                 bacc = balanced_accuracy_score(true, pred)
-                clr = classification_report(true, pred)
-                cfm = confusion_matrix(true, pred)
                 r2 = r2_score(true, pred)
                 try:
                     roc = roc_auc_score(true, pred)
                 except ValueError:
                     roc = None
 
-                if self.target_class == 'binary':
+                if target_class == 'binary':
                     f1 = f1_score(true, pred)
                     pre = precision_score(true, pred)
                     rec = recall_score(true, pred)
 
-                elif self.target_class == 'multiclass':
+                elif target_class == 'multiclass':
                     if self.imbalanced is True:
                         f1 = f1_score(true, pred, average='micro')
                         pre = precision_score(true, pred, average='micro')
@@ -741,8 +733,6 @@ class MultiClassifier:
 
                 tacc = accuracy_score(true_train, pred_train)
                 tbacc = balanced_accuracy_score(true_train, pred_train)
-                tclr = classification_report(true_train, pred_train)
-                tcfm = confusion_matrix(true_train, pred_train)
                 tr2 = r2_score(true_train, pred_train)
                 try:
                     troc = roc_auc_score(true_train, pred_train)
@@ -750,12 +740,12 @@ class MultiClassifier:
 
                     troc = None
 
-                if self.target_class == 'binary':
+                if target_class == 'binary':
                     tf1 = f1_score(true_train, pred_train)
                     tpre = precision_score(true_train, pred_train)
                     trec = recall_score(true_train, pred_train)
 
-                elif self.target_class == 'multiclass':
+                elif target_class == 'multiclass':
                     if self.imbalanced is True:
                         tf1 = f1_score(true_train, pred_train, average='micro')
                         tpre = precision_score(true_train, pred_train, average='micro')
@@ -779,35 +769,37 @@ class MultiClassifier:
                     eval_mul = [overfit, tacc, acc, tbacc, bacc, tr2, r2, tf1, f1, tpre, pre, trec, rec,
                                 time_taken]
 
-                if self.target_class == 'binary':
+                if target_class == 'binary':
                     dataframe.update({names[i]: eval_bin})
-                elif self.target_class == 'multiclass':
+                elif target_class == 'multiclass':
                     dataframe.update({names[i]: eval_mul})
 
             if show_train_score is False:
-                if self.target_class == 'binary':
+                if target_class == 'binary':
                     df = pd.DataFrame.from_dict(dataframe,
                                                 orient='index',
                                                 columns=self.t_split_binary_columns_test)
 
-                elif self.target_class == 'multiclass':
+                elif target_class == 'multiclass':
                     df = pd.DataFrame.from_dict(dataframe,
                                                 orient='index',
                                                 columns=self.t_split_multiclass_columns_test)
 
             elif show_train_score is True:
-                if self.target_class == 'binary':
+                if target_class == 'binary':
                     df = pd.DataFrame.from_dict(dataframe,
                                                 orient='index',
                                                 columns=self.t_split_binary_columns_train)
 
-                elif self.target_class == 'multiclass':
+                elif target_class == 'multiclass':
                     df = pd.DataFrame.from_dict(dataframe,
                                                 orient='index',
                                                 columns=self.t_split_multiclass_columns_train)
 
             if return_best_model is not None:
                 logger.info(f'BEST MODEL BASED ON {return_best_model}')
+                retrieve_df = df.reset_index()
+                logger.info(f'The best model based on the {return_best_model} metric is {retrieve_df["index"][0]}')
                 display(df.sort_values(by=return_best_model, ascending=False))
 
             elif return_best_model is None:
@@ -822,7 +814,7 @@ class MultiClassifier:
             KFoldModel = self._initialize_()
             names = self.classifier_model_names()
 
-            if self.target_class == 'binary':
+            if target_class == 'binary':
                 logger.info("Training started")
                 dataframe = self._startKFold_(param=KFoldModel,
                                               param_X=X,
@@ -843,7 +835,7 @@ class MultiClassifier:
                 kf_ = kf_best_model(df, return_best_model, excel)
                 return kf_
 
-            elif self.target_class == 'multiclass':
+            elif target_class == 'multiclass':
                 logger.info("Training started")
                 dataframe = self._startKFold_(param=KFoldModel,
                                               param_X=X,
@@ -1019,7 +1011,7 @@ class MultiClassifier:
 
         names = self.classifier_model_names()
         sns.set()
-
+        target_class = _check_target(the_y)
         param['model_names'] = names
         FILE_FORMATS = ['pdf', 'png']
         if save not in FILE_FORMATS:
@@ -1087,7 +1079,7 @@ class MultiClassifier:
             display(plot1)
 
         elif t_split is True:
-            if self.target_class == 'binary':
+            if target_class == 'binary':
                 plt.figure(figsize=size)
                 plot = sns.barplot(x="model_names", y="Accuracy", data=param)
                 plot.set_xticklabels(plot.get_xticklabels(), rotation=90)
@@ -1132,7 +1124,7 @@ class MultiClassifier:
                     name = save_name
                     img(FILENAME=name, FILE_PATH=file_path, type_='picture')
 
-            elif self.target_class == 'multiclass':
+            elif target_class == 'multiclass':
                 plt.figure(figsize=size)
                 plot = sns.barplot(x="model_names", y="Accuracy", data=param)
                 plot.set_xticklabels(plot.get_xticklabels(), rotation=90)
@@ -1196,14 +1188,13 @@ class MultiClassifier:
                 """
 
         names = self.classifier_model_names()
-
         param['model_names'] = names
-
+        target_class = _check_target(the_y)
         if kf is True:
             if t_split is True:
                 raise Exception("set kf to True if you used KFold or set t_split to True"
                                 "if you used the split method.")
-            if self.target_class == 'binary':
+            if target_class == 'binary':
                 IMAGE_COLUMNS = []
                 for i in range(len(self.kf_binary_columns_train)):
                     IMAGE_COLUMNS.append(self.kf_binary_columns_train[i] + ".png")
@@ -1226,12 +1217,12 @@ class MultiClassifier:
                             img_plotly(
                                 name=IMAGE_COLUMNS[j],
                                 figure=fig,
-                                label=self.target_class,
+                                label=target_class,
                                 FILENAME=dire,
                                 FILE_PATH=file_path,
                             )
 
-            elif self.target_class == 'multiclass':
+            elif target_class == 'multiclass':
                 IMAGE_COLUMNS = []
                 for i in range(len(self.kf_multiclass_columns_train)):
                     IMAGE_COLUMNS.append(self.kf_multiclass_columns_train[i] + ".png")
@@ -1254,7 +1245,7 @@ class MultiClassifier:
                             img_plotly(
                                 name=IMAGE_COLUMNS[j],
                                 figure=fig,
-                                label=self.target_class,
+                                label=target_class,
                                 FILENAME=dire,
                                 FILE_PATH=file_path,
                             )
@@ -1264,7 +1255,7 @@ class MultiClassifier:
                 raise Exception("set kf to True if you used KFold or set t_split to True"
                                 "if you used the split method.")
 
-            if self.target_class == 'binary':
+            if target_class == 'binary':
                 IMAGE_COLUMNS = []
                 for i in range(len(self.t_split_binary_columns_test)):
                     IMAGE_COLUMNS.append(self.t_split_binary_columns_test[i] + ".png")
@@ -1287,12 +1278,12 @@ class MultiClassifier:
                             img_plotly(
                                 name=IMAGE_COLUMNS[j],
                                 figure=fig,
-                                label=self.target_class,
+                                label=target_class,
                                 FILENAME=dire,
                                 FILE_PATH=file_path,
                             )
 
-            elif self.target_class == 'multiclass':
+            elif target_class == 'multiclass':
                 IMAGE_COLUMNS = []
                 for i in range(len(self.t_split_multiclass_columns_test)):
                     IMAGE_COLUMNS.append(self.t_split_multiclass_columns_test[i] + ".png")
@@ -1315,7 +1306,7 @@ class MultiClassifier:
                             img_plotly(
                                 name=IMAGE_COLUMNS[j],
                                 figure=fig,
-                                label=self.target_class,
+                                label=target_class,
                                 FILENAME=dire,
                                 FILE_PATH=file_path,
                             )
